@@ -1,6 +1,6 @@
 //-*-c++-*-
 //
-//  $Id: spamass-milter.h,v 1.8 2002/12/23 17:08:30 dnelson Exp $
+//  $Id: spamass-milter.h,v 1.17 2003/06/14 19:17:41 dnelson Exp $
 //
 //  Main include file for SpamAss-Milter
 //
@@ -25,11 +25,14 @@
 //   Contact:
 //            Michael Brown <michaelb@opentext.com>
 //
+#include <list>
 
 using namespace std;
 
 string retrieve_field(const string&, const string&);
 
+sfsistat mlfi_connect(SMFICTX*, char*, _SOCK_ADDR*);
+sfsistat mlfi_helo(SMFICTX*, char*);
 sfsistat mlfi_envrcpt(SMFICTX*, char**);
 sfsistat mlfi_envfrom(SMFICTX*, char**);
 sfsistat mlfi_header(SMFICTX*, char*, char*);
@@ -42,6 +45,28 @@ sfsistat mlfi_abort(SMFICTX*);
 
 extern struct smfiDesc smfilter;
 
+/* struct describing a single network */
+struct net
+{
+	struct in_addr network;
+	struct in_addr netmask;
+};
+
+/* an array of networks */
+struct networklist
+{
+	struct net *nets;
+	int num_nets;
+};
+
+
+// Debug tokens.
+enum debuglevel 
+{
+	D_ALWAYS, D_FUNC, D_POLL, D_UORI, D_STR, D_MISC, D_NET, D_SPAMC,
+	D_MAX // must be last
+};
+
 class SpamAssassin {
 public:
   SpamAssassin();
@@ -49,6 +74,8 @@ public:
 
   void Connect();
   void output(const void*, long);
+  void output(const void*);
+  void output(string);
   void close_output();
   void input();
 
@@ -63,6 +90,8 @@ public:
   string& content_type();
   string& subject();
   string& rcpt();
+  string& from();
+  string& connectip();
   string  local_user();
   int     numrcpt();
   int     set_numrcpt();
@@ -76,6 +105,8 @@ public:
   string::size_type set_content_type(const string&);
   string::size_type set_subject(const string&);
   string::size_type set_rcpt(const string&);
+  string::size_type set_from(const string&);
+  string::size_type set_connectip(const string&);
 
 private:
   void empty_and_close_pipe();
@@ -85,23 +116,43 @@ public:
   // flags
   bool error;
   bool running;
-  bool connected;
+  bool connected;	/* are we connected to spamc? */
 
   // This is where we store the mail after it
   // was piped through SpamAssassin
   string mail;
 
+  // Data written via output() but before Connect() is stored here
+  string outputbuffer;
+
   // Variables for SpamAssassin influenced fields
   string x_spam_status, x_spam_flag, x_spam_report, x_spam_prev_content_type;
-  string x_spam_checker_version, x_spam_level, _content_type, _subject, _rcpt;
+  string x_spam_checker_version, x_spam_level, _content_type, _subject;
+  
+  // Envelope info: MAIL FROM:, RCPT TO:, and IP address of remote host
+  // _rcpt only holds the first recipient if there are more than one
+  string _from, _rcpt, _connectip;
   
   // Counter to keep track of the number of recipients
   int    _numrcpt;
 
+  // The list of recipients for the current message
+  list <string> recipients;
+
+  // the sendmail queue id for this message; used for logging
+  string queueid;
+
   // Process handling variables
   pid_t pid;
   int pipe_io[2][2];
+};
   
+/* Private data structure to carry per-client data between calls */
+struct context
+{
+	struct in_addr connect_ip;	// remote IP address
+	char *helo;
+	SpamAssassin *assassin; // pointer to the SA object if we're processing a message
 };
 
 /* This hack is the only way to call pointers to member functions! */
@@ -111,7 +162,11 @@ typedef string::size_type (SpamAssassin::*t_setter)(const string &val);
 int assassinate(SMFICTX*, SpamAssassin*);
 
 void throw_error(const string&);
-void debug(int level, const char* string, ...);
+void debug(enum debuglevel, const char* fmt, ...); /* __printflike(2, 3); */
 string::size_type find_nocase(const string&, const string&, string::size_type = 0);
 int cmp_nocase_partial(const string&, const string&);
 void closeall(int fd);
+void parse_networklist(char *string, struct networklist *list);
+int ip_in_networklist(struct in_addr ip, struct networklist *list);
+void parse_debuglevel(char* string);
+char *strlwr(char *str);
